//-----------------------------------------------------------------------------
//
// Source File Name: pwCCSecExit.java
// Version: 1.0
// Description: MQSecurityExit implementation
//
// This is a sample Control Center security exit which should
// be used in conjuntion with the sample queue manager security exit
// pwQMSecExit.c
//
//-----------------------------------------------------------------------------


import java.io.*;
import java.util.*;
import com.ibm.mq.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;


//-----------------------------------------------------------------------------
// MQSISecurityExit class
//-----------------------------------------------------------------------------
//Implementation of the MQSecurityExit interface
//-----------------------------------------------------------------------------
                                                      
public class pwCCSecExit implements MQSecurityExit

{
  
  //---------------------------------------------------------------------------
  // Variable definitions
  //---------------------------------------------------------------------------
     char[] recPassword = null;
          
    
  //--------------------------------------------------------------------------
  // Method: securityExit
  //--------------------------------------------------------------------------
  //Implement the securityExit method
  //--------------------------------------------------------------------------

  public byte[] securityExit(MQChannelExit       channelExitParms,
                             MQChannelDefinition channelDefinition,
                             byte[]              agentBuffer)
  {
    String userName;
    byte[] userBytes;
    String pswdPref = "pswd";
    String usidPref = "usid";
    String agentData = null;
    //expected data from queue manager exit
    String expRecMsg = "userid received\0";
    byte[] expRecMsgBytes = null;
    String passwordToSend = null;
    boolean result = true;

    
     
    //determine reason for invocation
    switch (channelExitParms.exitReason) 
    {
      //----------------------------------------------------------------------
      // MQXR_INIT: exit initialization - return MQXCC_OK 
      //----------------------------------------------------------------------
      case MQChannelExit.MQXR_INIT:
        
        channelExitParms.exitResponse = MQChannelExit.MQXCC_OK;
        
      break;                                            

      //----------------------------------------------------------------------
      // MQXR_INIT_SEC: Initiate message exchange by returning 
      // MQXCC_SEND_AND_REQUEST_SEC_MSG and sending client userid
      //----------------------------------------------------------------------
      case MQChannelExit.MQXR_INIT_SEC:
         
         //get the client userid
         userName = System.getProperty("user.name");
         //reset agentBuffer and send userid - add userid prefix so queue manager exit
         //identifies as a userid
         agentBuffer = null;
         agentData = usidPref + userName;
         //convert data to bytes and assign to agentBuffer
         try {
             agentBuffer = agentData.getBytes("UTF8");
         }
         catch (UnsupportedEncodingException e)
         {
             agentBuffer = null;
         }

         //return MQXCC_SEND_AND_REQUEST_SEC_MSG - sending message in agentBuffer and 
         //expecting response from queue manager exit
         channelExitParms.exitResponse = MQChannelExit.MQXCC_SEND_AND_REQUEST_SEC_MSG;
            
      break;

      //----------------------------------------------------------------------
      // MQXR_SEC_MSG: receive message from queue manager exit and then send
      // client id password
      //----------------------------------------------------------------------
      case MQChannelExit.MQXR_SEC_MSG:
        
          if (agentBuffer.length == 0)
          {
            //if no data sent from queue manager exit then close channel
            System.out.println("ERROR! no data in inbound agentBuffer, closing channel");
            channelExitParms.exitResponse = MQChannelExit.MQXCC_SUPPRESS_FUNCTION;
          }
          else
          {//convert expected message to bytes
             try
             {
               expRecMsgBytes = expRecMsg.getBytes("UTF8");
               
             }
             catch  (UnsupportedEncodingException e)
             {
               System.out.println("ERROR!! - Failed to convert received data\n");
             }
             
             //check if message received in agentBuffer matches expected message
             for (int i=0; i < agentBuffer.length; i++ ) {
                 if (agentBuffer[i] != expRecMsgBytes[i]) {
                     result = false;
                 }
             }


             if (result)
             {
                //prompt for client password
                getPswd();
                //convert password to string
                passwordToSend = new String(recPassword);
                //add password prefix
                agentData = pswdPref + passwordToSend;
                //convert to bytes and assign to agentBuffer
                try
                {
                  agentBuffer = agentData.getBytes("UTF8"); 
                }
                catch (UnsupportedEncodingException e)
                {
                  agentBuffer = null;
                }
	
                //return SEND_SEC_MSG to send message in agentBuffer
                channelExitParms.exitResponse = MQChannelExit.MQXCC_SEND_SEC_MSG;

             }	
             else
             {
                //inbound agentBuffer does not match expected message so close channel
                System.out.println("ERROR! inbound agentBuffer not match expected message, closing channel");
                channelExitParms.exitResponse = MQChannelExit.MQXCC_SUPPRESS_FUNCTION;
             }
          
             //reset password fields
             recPassword = null;
             passwordToSend = null;
                    
          }

      break;
      
      //----------------------------------------------------------------------
      // MQXR_TERM: 
      //----------------------------------------------------------------------
      case MQChannelExit.MQXR_TERM:
        
        //return MQXCC_OK at exit termination               
        channelExitParms.exitResponse = MQChannelExit.MQXCC_OK;
      
      break;

      //----------------------------------------------------------------------
      // Unexpected call: Close the connection.
      //----------------------------------------------------------------------
      default:
         
         System.out.println("ERROR!! - Invoked with unexpected reason!!");
         channelExitParms.exitResponse = MQChannelExit.MQXCC_SUPPRESS_FUNCTION;
      
      break;
    }
    
    
    
    //------------------------------------------------------------------------
    // Always return the buffer of data passed in.                  
    //------------------------------------------------------------------------
          
     return agentBuffer;
     
   }

public void getPswd()
{
            
      // Create Frame and centre the dialog from it
      final Frame appFrame = new Frame();
      // Open a dialog and set it modal
      final JDialog dialog = new JDialog(appFrame, "Password Prompt", true);
      JLabel label = new JLabel("Enter your password  ");
      JPasswordField passwordField = new JPasswordField(20);
      passwordField.setEchoChar('*');
      //create action listener for password input
      passwordField.addActionListener(new ActionListener() {
      public void  actionPerformed(ActionEvent e) {
                     JPasswordField input = (JPasswordField)e.getSource();
                     recPassword = input.getPassword();
                     //once password read reset password field to nulls
                     input = null;
                     //Close the dialog when user hits Enter
    		     dialog.dispose();
                     }
      });  //end of addActionListener
      JPanel contentPane = new JPanel(new BorderLayout());
      contentPane.setBorder(BorderFactory.createEmptyBorder(20, 20, 20, 20));
      contentPane.add(label, BorderLayout.WEST);
      contentPane.add(passwordField, BorderLayout.CENTER);
      //set dialog size and center dialog
      dialog.setContentPane(contentPane);
      dialog.addWindowListener(new WindowAdapter() {
        public void windowClosing(WindowEvent e) { 
        }
      });
      dialog.pack();
      dialog.setSize(400,90);
      dialog.setLocationRelativeTo(appFrame);
      dialog.setVisible(true);
                
      
}



}
                           



